/*
 * kylin-status-manager
 *
 * Copyright (C) 2022, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: wangguixi <wangguixi@kylinos.cn>
 *
 */
#include "bluez_keyboard_monitoring.h"

Bluez_Keyboard_Monitoring::Bluez_Keyboard_Monitoring()
{
    qDebug()<<"蓝牙键盘监控";
    if(QGSettings::isSchemaInstalled("org.ukui.bluetooth")) {
        gsetting = new QGSettings("org.ukui.bluetooth");
        Default_Adapter = gsetting->get("adapter-address").toString();
        qDebug()<<"Default_Adapter"<<Default_Adapter;
    }
    this->start();
//    start_to_monitor_blue_keyboard();
}
Bluez_Keyboard_Monitoring::~Bluez_Keyboard_Monitoring()
{
    if(manager) {
        delete manager;
    }
    if(job) {
        delete job;
    }
    if(gsetting) {
        delete gsetting;
    }
}
void Bluez_Keyboard_Monitoring::run()
{
    int times=0;
    while(true)
    {
        manager=new BluezQt::Manager(this);
        if(manager==nullptr)
        {
            qDebug()<<"manager is null";
            return ;
        }
        job=manager->init();
        job->exec();
        if(!manager->isOperational()) {
            qWarning("Bluze is not running");
            delete manager;
            manager =nullptr;
            qWarning("Bluze restart");
            qDebug()<<"sleep";
            QThread::sleep(5);
            if(times>=10) {
                qDebug()<<"This machine may not have Bluetooth";
                return ;
            }
            times=times+1;
        } else {
            qDebug()<<"start";
            break;
        }
    }
    start_to_monitor_blue_keyboard();
}
BluezQt::AdapterPtr Bluez_Keyboard_Monitoring::get_Defualt_Adapter()
{
    qDebug()<<manager->adapters().size();
    if(manager->adapters().size() == 0) {
        return nullptr;
    }else if (manager->adapters().size() == 1) {
        return manager->adapters().at(0);
    }else{
        if(gsetting) {
            Default_Adapter = gsetting->get("adapter-address").toString();
            return manager->adapterForAddress(Default_Adapter);
        }
        qDebug()<<"多个";
        return nullptr;
    }
}
void Bluez_Keyboard_Monitoring::connect_To_Adapter()
{
    blqadaoter.data()->disconnect();
    connect(blqadaoter.data(),&BluezQt::Adapter::deviceChanged,this,[this](BluezQt::DevicePtr devPtr){
        if(devPtr->type()==BluezQt::Device::Type::Keyboard) {
            qDebug()<<"deviceChanged Type::Keyboard";
            Q_EMIT blue_keyboard_ststus_changed(2);
        }
    });
}
void Bluez_Keyboard_Monitoring::start_to_monitor_blue_keyboard()
{
    if(!manager->isInitialized()) {
        qWarning("BluezQt::Manager init falied!!");
        return ;
    }
    //========适配器变化===重新更新====
    connect(manager,&BluezQt::Manager::adapterAdded,this,[=](BluezQt::AdapterPtr adpptr)  {
        qDebug()<<"manager adapterAdded-------------------------------";
        blqadaoter = get_Defualt_Adapter();
        connect_To_Adapter();
    });
    connect(manager,&BluezQt::Manager::adapterChanged,this,[=](BluezQt::AdapterPtr adpptr) {
        qDebug()<<"manager adapterChanged------------------------------";
        blqadaoter = get_Defualt_Adapter();
        connect_To_Adapter();
    });
    connect(manager,&BluezQt::Manager::adapterRemoved,this,[=](BluezQt::AdapterPtr adpptr) {
        qDebug()<<"manager adapterRemoved--------------------------------";
        blqadaoter = get_Defualt_Adapter();
        connect_To_Adapter();
    });

    //初始化
    blqadaoter = get_Defualt_Adapter();
    connect_To_Adapter();
    qDebug()<<"connectedChanged end";
}
