/*
 * kylin-status-manager
 *
 * Copyright (C) 2022, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: wangguixi <wangguixi@kylinos.cn>
 *
 */
#include "builtin_keyboard_monitoring.h"
// 状态管理器
#define SETTINGS_KYLINSTATUSMANAGER    "org.kylin-status-manager"
#define MODE_MONITER         "modemonitor" // 是否自动切换
#define TABLET_MODE          "tablet-mode"

Builtin_Keyboard_monitoring::Builtin_Keyboard_monitoring(QObject *parent) : QObject(parent)
{
    //有AT Translated Set 2 keyboard 字段
    //获得键盘型号，通过机器类型配置文件
    QString path="/etc/ukui/kylin-status-manager/statusconfig.ini";
    setting=new QSettings(path,QSettings::IniFormat);
    setting->beginGroup("type");
    QString type= setting->value("productname").toString();
    setting->endGroup();
    setting->sync();

    setting->beginGroup("tabletlist");
    QStringList tabletlist =setting->allKeys();
    setting->endGroup();
    setting->sync();
    qDebug()<<"平板设备的列表"<<tabletlist;
    qDebug()<<"当前设备的型号"<<type;
    for(int i=0; i<tabletlist.count();i++){
        if(tabletlist.at(i)==type){
            qDebug()<<"机器类型为没有内置键盘的平板";
            builtin =false;
            return ;
        }
    }
    if(type.contains("HP Tablet 11-be")){
        qDebug()<<"机器类型为没有内置键盘的HP Tablet 11-be系列平板";
        builtin =false;
        return ;
    }
    if(type == "HP Tablet 11-be0xxx"||type == "FT-2000/4"||type == "X6-PRO") {//mavis机器//如果适配其他机器需要对新机器的情况进行适配
        qInfo()<<"内置键盘不存在"<<"整机型号："<<type;
        builtin =false;//builtin 内置键盘标志位
    } else {
        //有内置键盘，则检查传感器
        //监控dbus
        qInfo()<<"内置键盘存在"<<"整机型号："<<type;
        builtin =true;
        if(iiosensor_matching1()) {
            qDebug()<<"内核传感器匹配成功";
            iiosensor_matchingnum=1;
            return ;
        }

        if(iiosensor_matching2()) {
            qDebug()<<"ksm传感器匹配成功";
            iiosensor_matchingnum=2;
            return ;
        }
        qInfo()<<"传感器匹配不成功，当前机器没有找到传感器";
    }
}

Builtin_Keyboard_monitoring::~Builtin_Keyboard_monitoring()
{
    delete setting;
    if(status_manager_Setting) {
        delete status_manager_Setting;
    }
    if(iioIface1) {
        delete iioIface1;
    }
    if(plmIface1) {
        delete plmIface1;
    }

    if(plmIface2) {
        delete plmIface2;
    }
}
bool Builtin_Keyboard_monitoring::iiosensor_matching1()
{
    iioIface1 = new QDBusInterface("com.kylin.iio.sensor.manager",
                                   "/com/kylin/iio/sensor/manager",
                                   "com.kylin.iio.sensor.manager",
                                    QDBusConnection::systemBus());
    if(!iioIface1){//异常处理
        qInfo()<<"内核iiosensor-manager dbus服务不存在";
        return 0;
    }
    QDBusMessage result = iioIface1->call("GetDeviceList");
//    qDebug()<<"result"<<result;
    for (int i=0; i<result.arguments().size();i++){
         QString str = result.arguments().value(i).toString();
         qDebug()<<"str"<<str;
         if(str.contains("plm")) {
             QStringList list = str.split(":");
             str = list[1];
             qDebug()<<"str"<<str;
             QDBusConnection::systemBus().connect("com.kylin.iio.sensor.manager",
                                                  str,
                                                  "com.kylin.iio.sensor.manager.device.plm",
                                                  "AttrsChanged",
                                                  this,
                                                  SLOT(on_plm_changed1(QString,QDBusVariant)));
             plmIface1 = new QDBusInterface("com.kylin.iio.sensor.manager",
                                             str,
                                             "com.kylin.iio.sensor.manager.device.plm",
                                             QDBusConnection::systemBus());
             if (!plmIface1){
                 qInfo()<<"内核iiosensor-manager plm（角度）传感器 is null";
                 return 0;
             }
             return 1;
//             break;
         }else{
             qInfo()<<"内核iiosensor-manager 不存在plm（角度）传感器";
             return 0;
         }
    }
    return 0;
}
bool Builtin_Keyboard_monitoring::iiosensor_matching2()
{
    plmIface2 = new QDBusInterface("com.kylinstatusmanager.interface",
                                  "/",
                                  "com.kylinstatusmanager.interface",
                                    QDBusConnection::systemBus());
    if(!plmIface2){//异常处理
        qInfo()<<"kylinstatusmanager iiosensor dbus服务不存在";
        return 0;
    }
    QDBusConnection::systemBus().connect("com.kylinstatusmanager.interface",
                                         "/",
                                         "com.kylinstatusmanager.interface",
                                         "solding_state_changed_signal",
                                         this,
                                         SLOT(on_plm_changed2(int)));

    QDBusReply<QString> result = plmIface2->call("get_current_solding_state");

    QString now=result.value();
    qDebug()<<now;
    if(now=="null"||now==""){
        return 0;
    }else {
      return 1;
    }


}
bool Builtin_Keyboard_monitoring::get_builtin_keyboard_status()
{
    //有内置
    if(builtin) {
        if(iiosensor_matchingnum == 1){
            QDBusReply<int> result = plmIface1->call("GetAttrValue","in_lidmode_raw");
            quint32 now = result.value();
            if(now == 1|| now == 0){
                return 1;
            } else {
                return 0;
            }
        }else if(iiosensor_matchingnum == 2){
            QDBusReply<QString> result = plmIface2->call("get_current_solding_state");
            QString now=result.value();
            qDebug()<<now;
            if (now =="Clamshell") {
               return 1;
            }else {
               return 0;
            }
        }else{
            //关闭自动切换
            QByteArray id(SETTINGS_KYLINSTATUSMANAGER);
            if(QGSettings::isSchemaInstalled(id)) {
                status_manager_Setting=new QGSettings(id);
                status_manager_Setting->set(MODE_MONITER,false);
            } else {
                qInfo()<<"org.kylin-status-manager is not install";
            }
            return 1;//没匹配到应该默认关闭自动切换，并认为键盘可用
        }
    }else {
    //没有内置
        return 0;
    }
}
void Builtin_Keyboard_monitoring::on_plm_changed1(QString key ,QDBusVariant status)
{
    qDebug()<<"key"<<key;
    if (key == "in_lidmode_raw") {//in_lidmode_raw
//        QVariant vFirst = status.variant();
//        quint32 now = vFirst.toUInt();
//        if(now == 1){
//           builtin_enable = true;
//        } else if (now == 0) {
//           builtin_enable = true;
//        } else {
//            builtin_enable = false;
//        }
//        qDebug()<<"status"<<vFirst;
//        qDebug()<<"quint32"<<now;
//只需要是那个字段改变的信号，具体的值不重要，在监控信号哪会获取当前最新的。(暂时函数调用有问题，只能使用信号22-04-18)
        Q_EMIT builtin_keyboard_ststus_changed(1);
    }
}
void Builtin_Keyboard_monitoring::on_plm_changed2(int original_mode)
{

//    if((original_mode == 7)||(original_mode == 2)||
//            (original_mode == 3)||(original_mode == 4)||(original_mode == 5)){
//        Q_EMIT builtin_keyboard_ststus_changed(1);
//    }else {
//        Q_EMIT builtin_keyboard_ststus_changed(1);
//    }
    Q_EMIT builtin_keyboard_ststus_changed(1);

}
