/*
 * kylin-status-manager
 *
 * Copyright (C) 2022, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: wangguixi <wangguixi@kylinos.cn>
 *
 */
#include "keyboard_status_monitoring.h"
#include <QThread>
Keyboard_Status_Monitoring::Keyboard_Status_Monitoring(QObject *parent) : QObject(parent)
{
    bkeyboard = new Bluez_Keyboard_Monitoring;
    ukeyboard = new Usb_Keyboard_Monitoring;
    connect(bkeyboard,&Bluez_Keyboard_Monitoring::blue_keyboard_ststus_changed,this,
            &Keyboard_Status_Monitoring::get_keyboard_ststus_result);
    connect(ukeyboard,&Usb_Keyboard_Monitoring::usb_keyboard_ststus_changed,this,
            &Keyboard_Status_Monitoring::get_keyboard_ststus_result);
    if(is_ATTranslatedSet2keyboard()){
        bikeyboard = new Builtin_Keyboard_monitoring;
        connect(bikeyboard,&Builtin_Keyboard_monitoring::builtin_keyboard_ststus_changed,this,
                &Keyboard_Status_Monitoring::get_keyboard_ststus_result);
    }
}
Keyboard_Status_Monitoring::~Keyboard_Status_Monitoring()
{
    delete bkeyboard;
    delete ukeyboard;
    if(bikeyboard) {
        delete bikeyboard;
    }
}
//获取设备中的键盘设备列表
QStringList Keyboard_Status_Monitoring::get_keyboardlist()
{
    QProcess process;
    QStringList options;
    options << "-c";
    options << "cat /proc/bus/input/devices |grep -i keyboard";
    process.start("/bin/bash",options);
    process.waitForFinished();
    QString result = process.readAllStandardOutput();
    QStringList list = result.split("\"");
    QStringList keyboardlist;
    for(int i=0;i<=list.count()-1;i++){
        QString str=list.at(i);
        if(str.contains("keyboard",Qt::CaseInsensitive)) {
            keyboardlist.append(str);
        }
    }
    return keyboardlist;
}
int Keyboard_Status_Monitoring::get_keyboard_ststus_result(int type)// 0 初始化 1 内置 2 蓝牙 3 usb 4查看键盘状态
{

    bool sendsignal =true;//默认调用该函数发送键盘状态信号，false不发送
    if(type == 4){
        sendsignal=false;
    }
    //加延迟，解决蓝牙连接时过早读取的设备导致设备列表读取不准确的情况
    if(type ==2) {
        QThread::msleep(100);
    }

    int enable = -1;//键盘是否可用返回值，默认-1，可用1，不可用0
    QStringList keyboardlist;
    keyboardlist=get_keyboardlist();

    //如果键盘状态是usb改变则要与上一次的情况作对比，过滤不是键盘的usb设备
    if(type == 3) {
        if(last_keyboard_list == keyboardlist) {
            return -1;
        }
    }

    if(!bikeyboard) {
        //为空,没有内置键盘字段,说明获取的键盘列表有就说明物理键盘可用
        if(keyboardlist.count()>=1) {
            enable=1;//有键盘
        } else {
            enable=0;//无键盘
        }
    }else {
        //不为空，说明有内置键盘字段，但不一定该设备的内置键盘就是物理键盘还需要根据机器区分是是否是功能按钮
        //内置键盘判断中已经处理好了

        if(keyboardlist.count()>=2) {
            //大于2说明不管内置键盘能不能用，至少还有一个外接的外设键盘可用
            enable=1;//有键盘
        }else {
            //只有一个说明只有内置键盘
            if(bikeyboard->get_builtin_keyboard_status()) {
                //内置键盘可用
                enable=1;//有键盘
            }else {
                enable=0;//无键盘
            }
        }
    }
    last_keyboard_list=keyboardlist;
    //有键盘可用则发送有键盘信号。 无键盘可用则发无键盘信号
    if (sendsignal) {
        if(enable == 1) {
            Q_EMIT keyboard_enable_signal(true);
        } else {
            Q_EMIT keyboard_enable_signal(false);
        }
    }
    return enable;
}
//判断是否有内置键盘字段
bool Keyboard_Status_Monitoring::is_ATTranslatedSet2keyboard()
{
    QStringList keyboardlist;
    keyboardlist=get_keyboardlist();
    if(keyboardlist.contains("AT Translated Set 2 keyboard")) {
        //有 AT Translated Set 2 keyboard 字段
        return 1;
    }else {
        //没有 AT Translated Set 2 keyboard 字段
        return 0;
    }
}
