/*
 * kylin-status-manager
 *
 * Copyright (C) 2022, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: wangguixi <wangguixi@kylinos.cn>
 *
 */
#include "usb_keyboard_monitoring.h"

Usb_Keyboard_Monitoring::Usb_Keyboard_Monitoring()
{
    qDebug()<<"usb 热插拔开始监控";
    this->start();
}
Usb_Keyboard_Monitoring::~Usb_Keyboard_Monitoring()
{

}
void Usb_Keyboard_Monitoring::run()
{
    monitor();
}

bool Usb_Keyboard_Monitoring::monitor()
{
    struct udev *udev;
    struct udev_monitor *udev_monitor;

    int ret, fdcount;
    fd_set readfds;
    struct udev_device *device;

    const char *addaction="add";
    const char *removeaction="remove";
//    const char *type="Keyboard";


    udev=udev_new();

    if (udev == NULL) {
            return 1;
    }

    udev_monitor = udev_monitor_new_from_netlink(udev, "udev");
    if (udev_monitor == NULL) {
            return 1;
    }

    /* Add some filter */
    udev_monitor_filter_add_match_subsystem_devtype(udev_monitor, "usb", NULL);
//    udev_monitor_filter_add_match_subsystem_devtype(udev_monitor, "block", NULL);

    ret = udev_monitor_enable_receiving(udev_monitor);
    if (ret < 0) {
            return 1;
    }
//    bool isinit=true;
    /* Event Loop */
    while (1)
    {
        FD_ZERO(&readfds);

        if (udev_monitor != NULL) {
                FD_SET(udev_monitor_get_fd(udev_monitor), &readfds);
        }

        fdcount = select(udev_monitor_get_fd(udev_monitor) + 1,
                                         &readfds, NULL, NULL, NULL);
        if (fdcount < 0) {
                if (errno != EINTR) {
                        return 1;
                }
                continue;
        }
        if (udev_monitor == NULL) {
                continue;
        }
        if (!FD_ISSET(udev_monitor_get_fd(udev_monitor), &readfds)) {
                continue;
        }
        device = udev_monitor_receive_device(udev_monitor);
        if (device == NULL) {
                continue;
        }

        const char* action=udev_device_get_property_value(device,"ACTION");
//        const char* id_model=udev_device_get_property_value(device,"ID_MODEL_FROM_DATABASE");
        const char* DEVPATH=udev_device_get_property_value(device,"DEVPATH");

        if(action&&DEVPATH) {
//            qDebug()<<"比较"<<strcmp(action,addaction)<<strcmp(id_model,type);
            if(strcmp(action,addaction)==0) {
                Q_EMIT usb_keyboard_ststus_changed(3);
            }
            if(strcmp(action,removeaction)==0) {
                Q_EMIT usb_keyboard_ststus_changed(3);
            }
        }
        udev_device_unref(device);
    }

    udev_monitor_unref(udev_monitor);
    udev_unref(udev);
    return 0;

}
