/*
 * kylin-status-manager
 *
 * Copyright (C) 2022, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: wangguixi <wangguixi@kylinos.cn>
 *
 */
#include "rotation_monitoring.h"
#include <QDebug>
#include <QProcess>
#include <QCoreApplication>
#include <QTime>
#define SETTINGS_KYLINSTATUSMANAGER    "org.kylin-status-manager"
#define AUTO_ROTATION        "auto-rotation" // 旋转
#define ROTATIONS            "rotations"
Rotation_Monitoring::Rotation_Monitoring(QObject *parent) : QObject(parent)
{
    QByteArray id(SETTINGS_KYLINSTATUSMANAGER);
    if(QGSettings::isSchemaInstalled(id))
    {
        status_manager_Setting = new QGSettings(id);
        qDebug()<<"==========Rotation_Monitoring==========";
        start_sensor_monitoring();
    }else {
        qInfo()<<"org.kylin-status-manager is not install";
        return;
    }
}
Rotation_Monitoring::~Rotation_Monitoring()
{
    if(qsensor) {
        delete qsensor;
    }
    if(status_manager_Setting) {
        delete status_manager_Setting;
    }
}
void Rotation_Monitoring::start_sensor_monitoring()
{
    qDebug()<<"============start_sensor_monitoring==============";
    qsensor = new QOrientationSensor();
    connect(qsensor,&QOrientationSensor::readingChanged,this,&Rotation_Monitoring::rotations_change);
    if(qsensor->start()) {
        qDebug()<<"qsensor start successful";
    }else {
        qDebug()<<"qsensor start failt";
        int times=0;
        while (!qsensor->isActive()) {
            qsensor = new QOrientationSensor();
            connect(qsensor,&QOrientationSensor::readingChanged,this,&Rotation_Monitoring::rotations_change);
            qsensor->start();
            Sleep(500);
            times=times+1;
            if(times>=10){
                qInfo()<<"This machine may not have qsensor,can`t auto rotation";
                status_manager_Setting->set(AUTO_ROTATION,false);
                return ;
            }
            qInfo()<<"qsensor restart fail,after 1s restart sensor ";
        }
        qDebug()<<"qsensor restart successful";
    }
}
void Rotation_Monitoring::rotations_change()
{
    qDebug()<<"=================rotations_change=============";
    bool openRotationsMonitor =status_manager_Setting->get(AUTO_ROTATION).toBool();
    qDebug()<<"key is :autoRotation"<<openRotationsMonitor;
    if (!openRotationsMonitor) {//自动旋转打开
        return;
    }
    if(!qsensor->isActive()) {
        qInfo()<<"qsensor is not Active";
        return ;
    }
    int valueCount=qsensor->reading()->valueCount();
    if(valueCount == 0) {
        return ;
    }

    int statusnum=qsensor->reading()->value(0).toInt();
    QString rotations= "null";
    switch (statusnum) {
    case normal:
        rotations="normal";
        break;
    case upsidedown:
        rotations = "upside-down";
        break;
    case left:
        rotations = "left";
        break;
    case right:
        rotations ="right";
        break;
    default:
        break;
    }
    qDebug()<<"sensor return num is "<<statusnum<<"rotations status is "<<rotations;
//    if(status_manager_Setting) {
//        status_manager_Setting->set(ROTATIONS,rotations);
//    }
    Q_EMIT rotations_change_signal(rotations);

}
QString Rotation_Monitoring::get_rotation_status()
{
    qDebug()<<"get_rotation_status";
    if(!qsensor->isActive()) {
        qInfo()<<"qsensor is not Active";
            return "null";
    }
    int valueCount=qsensor->reading()->valueCount();
    if(valueCount == 0){
        qInfo()<<"qsensor->reading()->valueCount() 失败";
        return "normal";
    }
    int statusnum=qsensor->reading()->value(0).toInt();
    qDebug()<<"get_rotation_status"<<statusnum;
    QString rotations= "null";
    switch (statusnum) {
    case normal:
        rotations="normal";
        break;
    case upsidedown:
        rotations="upside-down";
        break;
    case left:
        rotations = "left";
        break;
    case right:
        rotations ="right";
        break;
    default:
        break;
    }
    return rotations;
}
void Rotation_Monitoring::Sleep(int msec)
{
    QTime dieTime = QTime::currentTime().addMSecs(msec);
    while( QTime::currentTime() < dieTime )
        QCoreApplication::processEvents(QEventLoop::AllEvents, 100);
}
