/*
 * kylin-status-manager
 *
 * Copyright (C) 2022, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: wangguixi <wangguixi@kylinos.cn>
 *
 */
#ifndef AUTOSWITCH_H
#define AUTOSWITCH_H

#include <QObject>
#include <QTimer>
#include <QTime>
#include <QSensor>
#include <QSettings>
#include <QGSettings>
#include "Devices_Status_Monitoring/devices_status_monitoring.h"


typedef struct _XDisplay Display;

class QTimer;
class QDBusInterface;

class autoSwitch : public QObject
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "com.kylin.statusmanager.interface")

public:
    explicit autoSwitch(QObject *parent = nullptr);
    ~autoSwitch();

    enum FoldStatus
    {
        normal=1,    //水平向下，不改变pc/table模式，不改变前一个信号值
        upsidedown=2,  //
        left=3,        //
        right=4 ,      //
    };
    Q_ENUM(FoldStatus)
public slots:
    int get_inputmethod_mode();                                           //获得当前键盘模式
    QString get_current_stylename();                                      //获得当前主题名字
    bool get_current_tabletmode();                                        //获得当前的模式
    QString get_current_rotation();                                       //获得当前旋转情况
    bool get_modemonitor();
    bool get_auto_rotation();
    bool set_rotation(QString rotation,QString who,QString why);
    bool set_tabletmode(bool tablet_mode,QString who,QString why);
    bool set_modemonitor(bool modemonitor,QString who,QString why);
    bool set_auto_rotation(bool autorotation,QString who,QString why);
    QString get_current_machine_category();

private:
    struct ksmData
    {
        QString rotation;
        bool tablet_mode;
        bool modemonitor;
        bool autorotation;
        bool keyboardenable;
    };
    ksmData ksmstruct;
    QSettings * setting = nullptr;     //机器型号判断
    //QGSettings
    QGSettings *ukui_style_Setting = nullptr;  //主题gsetting
    QGSettings *status_manager_Setting = nullptr;   //状态管理器的gsetting
    //防抖
    QTimer *debounce_timer=Q_NULLPTR; //计时器 、fold
    QTimer *debounce_timer1=Q_NULLPTR; //计时器、
    QTimer *debounce_timer2=Q_NULLPTR; //计时器、键盘
    int foldtime=0;
    int rotetime=0;
    int keyboardtim=0;
    //键盘检测
    Devices_Status_Monitoring * devicestatus;
    //-------------------------
    void init();
    void regist_dbus();
    void update_ksmstruct();
    void update_mode_status();
    void update_rotation_status();
    void update_keyboard_status();//更新键盘状态 //改为根据当前键盘情况更新状态 发送信号
    //-------------------------

signals:
    void inputmethod_change_signal(bool keyboard_enabled);                // 获取电脑状态 获取键盘禁用状态;false:禁用;
    void stylename_change_signal(QString style_name);                     // 主题更改信号
    void rotations_change_signal(QString rotations);                      // 旋转改变信号
    void mode_change_signal(bool tablet_mode);                            // 模式改变信号
    void modemonitor_change_signal(bool modemonitor);                     // 模式监控改变信号
    void auto_rotation_change_signal(bool auto_rotation);                 // 自动旋转改变信号
private slots:


     void mode_debounce_timeout();//防抖翻折
     void rotations_debounce_timeout();//防抖陀螺仪
     void keyboard_debounce_timeout();//防抖陀螺仪

     void mode_debounce(int num);
     void rotations_debounce(int num);
     void keyboard_debounce(int num);


     //-----------处理键盘enable信号 和旋转改变信号 Signal processing
     void processing_keyboard_enable_signal(bool enable);
     void processing_rotations_change_signal(QString rotations);
     //-----------
};

#endif // AUTOSWITCH_H
