/*
 * kylin-status-manager
 *
 * Copyright (C) 2022, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: wangguixi <wangguixi@kylinos.cn>
 *
 */
#include <QCoreApplication>
#include <QtDebug>
#include <QFile>
#include <QTextStream>
#include <QDateTime>
#include <QCommandLineOption>
#include <QCommandLineParser>
#include "Frontend_Console/autoswitch.h"
#include "Backend_Services/backend_init.h"
#include <ukui-log4qt.h>
void outputMessage(QtMsgType type, const QMessageLogContext &context, const QString &msg)
{
    QString txt;
    QString time = QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss");

    switch (type) {
      //调试信息提示
      case QtDebugMsg:
          txt = time + " " + QString("Debug: %1").arg(msg);
          break;
      //一般的warning提示
      case QtWarningMsg:
          txt = time + " " + QString("Warning: %1").arg(msg);
          break;
      //严重错误提示
      case QtCriticalMsg:
          txt = time + " " + QString("Critical: %1").arg(msg);
          break;
      //致命错误提示
      case QtFatalMsg:
          txt = time + " " + QString("Fatal: %1").arg(msg);
          abort();
      }

    QFile outFile("/tmp/kylin-status-manager-serivce.log");
    outFile.open(QIODevice::WriteOnly | QIODevice::Append);
    QTextStream ts(&outFile);
    ts << txt << endl;
}

int main(int argc, char *argv[])
{
    QCoreApplication a(argc, argv);

    QCommandLineParser parser;
    QCommandLineOption monitorRoleOption("service", "strat service");
    QCommandLineOption logOption("log", "open log");
    parser.addOption(monitorRoleOption);
    parser.addOption(logOption);
    parser.process(a);
    if(QCoreApplication::arguments().length()>1)
    {
        QString opt=QCoreApplication::arguments().at(1);
        if (opt == "--service") {
            qInstallMessageHandler(outputMessage);
            Backend_Init bi;
            return a.exec();
        }
        if(opt =="--log"||opt == "l"){
            autoSwitch au;
            return a.exec();
        }
    } else{
        initUkuiLog4qt("kylin-status-manager");
        autoSwitch au;
        return a.exec();
    }
}
